package CustomFieldEditor::CMS;

use strict;

use base 'Exporter';
our @EXPORT_OK = qw( rest_day is_limit );

use CustomFields::Util qw( get_meta save_meta field_loop _get_html );

use MT;
use MT::ListProperty;
use MT::Util;
use MT::Util qw( archive_file_for first_n_words encode_html epoch2ts format_ts relative_date encode_url );
use CustomFields::Field;
use CustomFields::App::CMS;
use Data::Dumper;

my $obj_types;
my $customfield_types;
my @fields = ( 'basename','default','description','name','obj_type','options','required','tag','type' );

sub init_request {
    my ($eh, $app) = @_;

    my $q = $app->param;
    my $mode = $q->param('__mode');
    return if !($mode eq 'list' || $mode eq 'filtered_list' || $mode eq 'save_cf_from_list' || $mode eq 'save');

    my $type = $q->param('_type') || $app->param('datasource');
    return if !$type eq 'field';

    my $blog_id = $q->param('blog_id') || 0;

    my $plugin = MT->component("CustomFieldEditor");
    my $field_column = $plugin->get_config_value('field_column', 'system');
    if (!$field_column) {
        $plugin->set_config_value('field_column', time(), 'system');
    }
    $plugin->set_config_value('rest_day', rest_day(), 'system');

    $obj_types = get_obj_types($app);
    $customfield_types = get_customfield_types($app);

    1;
}

sub bulk_sort {
    my $prop = shift;
    my ($objs) = @_;
    my $basename = $prop->basename;
    my %entry_id = map { $_->id => 1 } @$objs;
    my @entries = MT->model('entry')->load( { id => [ keys %entry_id ] } );
    my %data = map { $_->id => $_->meta('field.' . $basename) } @entries;
    return sort {
        $data{ $a->id } cmp $data{ $b->id }
    } @$objs;
}

sub out_html {
    my ( $prop, $obj, $app ) = @_;

    my $select = MT->translate('Select');
    my $delete = MT->translate('Delete');

    my $current_blog_id = $app->param('blog_id') || 0;
    my $blog_id = $obj->blog_id || 0;
    my $scope;
    my $scope_html;
    if ( !$current_blog_id || $blog_id != $current_blog_id ) {
        $scope = 'System';
        if ( $blog_id > 0 ) {
            my $blog = MT->model('blog')->load($blog_id);
            $scope = $blog->is_blog ? 'Blog' : 'Website';
        }
        my $scope_lc = lc $scope;
        my $scope_label = MT->translate($scope);
        $scope_html = qq{
            <span class="cf-scope $scope_lc sticky-label">$scope_label</span>
        };
    }





    my $plugin = MT->component("CustomFieldEditor");

    my $meta;
    my $key = $prop->id;
    $meta = $key =~ /type/ ? $plugin->translate( $obj->$key ) : $obj->$key;

    my $basename = $prop->id;

    if ($prop->id ne 'required') {
        $meta = '---' if !$meta;
    } else {
        $meta = '0' if !$meta;
    }
    my $id = $obj->id;

    my $asset_id;
    my $orig_meta;

    my $save_changes_label = MT->translate('Save Changes');
    my $save_label = MT->translate('Save');
    my $cancel_label = MT->translate('Cancel');
    my $element;
    if ( $prop->id eq 'name' || $prop->id eq 'basename' || $prop->id eq 'default' || $prop->id eq 'options' || $prop->id eq 'tag' ) {
        $element = qq{<input type="text" name="note-$basename-$id" id="note-$basename-$id" class="lfetext" value="} . encode_html($meta) . qq{" />};
    } elsif ( $prop->id eq 'description' ) {
        $element = qq{<textarea name="note-$basename-$id" id="note-$basename-$id" class="lfetextarea">} . encode_html($meta) . qq{</textarea>};
    } elsif ( $prop->id eq 'url' ) {
        $element = qq{<input type="text" name="note-$basename-$id" id="note-$basename-$id" class="lfeurl" value="} . encode_html($meta) . qq{" />};
    } elsif ( $prop->id eq 'required' ) {
        my $flag = $meta eq '0' ? '' : 'checked="checked"';
        $element = qq{<div><input type="checkbox" name="note-$basename-$id" id="note-$basename-$id" class="lfecheckbox" } . $flag . qq{ /></div>};
    } elsif ( $prop->id eq 'obj_type' || $prop->id eq 'type' ) {
        my $options = $prop->id eq 'obj_type' ? $obj_types : $customfield_types;
        my $buf;
        $buf .= qq{<option value=""></option>};
        for my $option (@{$options}) {
            my $label;
            my $key;
            my $type;
            if ( $prop->id eq 'obj_type' ) {
                $label = $option->{obj_type_label};
                $key = $option->{obj_type};
                $type = $obj->obj_type;
            } else {
                $label = $option->{label};
                $key = $option->{key};
                $type = $obj->type;
            }

            next if $label eq $plugin->translate('post_type');

            if ( $key eq $type ) {
                $buf .= qq{<option value="$key" selected="selected">$label</option>};
            } else {
                $buf .= qq{<option value="$key">$label</option>};
            }
        }
        $element = qq{<select name="note-$basename-$id" id="note-$basename-$id" class="lfeselect">} . $buf . qq{</select>};
#    } elsif ( $prop->id eq 'radio' ) {
#        my @options = split(',', $prop->option);
#        for my $option (@options) {
#            if ( $option eq $meta ) {
#                $element .= qq{<input type="radio" name="note-$basename-$id" value="$option" checked="checked" /> } . encode_html($option) . '<br />';
#            } else {
#                $element .= qq{<input type="radio" name="note-$basename-$id" value="$option" /> } . encode_html($option) . '<br />';
#            }
#        }
#        $element = qq{<div id="note-$basename-$id">} . $element . qq{</div>};
    }

    my $is_strict = 0;
    my $blog_id = $app->blog ? $app->blog->id : 0;
    $is_strict = $plugin->get_config_value('is_strict', 'blog:' . $app->blog->id ) if $blog_id;

    my $out;
    if ( $prop->id eq 'name' ) {
        my $edit_url  = MT->app->uri(
            mode => 'view',
            args => {
                _type   => 'field',
                id      => $obj->id,
                blog_id => $obj->blog_id,
            }
        );
        my $view_img = MT->static_path . 'images/nav_icons/mini/entries.gif';

        my $message = $app->translate('Edit Field');

        if ( ( $scope eq 'System' && $current_blog_id > 0 ) || ( $scope ne 'System' && $current_blog_id == 0 ) || ( $prop->id =~ /type/ && $is_strict ) ) {
            $out = $scope_html . encode_html($meta) . qq{<span id="note-$basename-field-$id" style="display: none">} . $element . qq{<span class="buttons"><a href="#$id" title="$save_changes_label" class="button submit-note">$save_label</a><a href="#$id" type="button" class="button cancel-edit">$cancel_label</a></span></span> <span class="view-link2"><a href="$edit_url" title="$message" target="_blank"><img alt="Edit Field" src="$view_img" style="vertical-align: unset;" /></a></span>};
        } else {
            $out = $scope_html . qq{<span id="note-$basename-link-$id" class="view-link"><a href="#$id" class="start-edit" title="}.encode_html($meta).qq{">}.encode_html($meta).qq{</a></span><span id="note-$basename-field-$id" style="display: none">} . $element . qq{<span class="buttons"><a href="#$id" title="$save_changes_label" class="button submit-note">$save_label</a><a href="#$id" type="button" class="button cancel-edit">$cancel_label</a></span></span> <span class="view-link2"><a href="$edit_url" title="$message" target="_blank"><img alt="Edit Field" src="$view_img" style="vertical-align: unset;" /></a></span>};
        }
    } else {
        if ( ( $scope eq 'System' && $current_blog_id > 0 ) || ( $scope ne 'System' && $current_blog_id == 0 ) || ( $prop->id =~ /type/ && $is_strict ) ) {
            $out = qq{<span id="note-$basename-link-$id" class="view-link">}.encode_html($meta).qq{</span><span id="note-$basename-field-$id" style="display: none">} . $element . qq{<span class="buttons"><a href="#$id" title="$save_changes_label" class="button submit-note">$save_label</a><a href="#$id" type="button" class="button cancel-edit">$cancel_label</a></span></span>};
        } else {
            $out = qq{<span id="note-$basename-link-$id" class="view-link"><a href="#$id" class="start-edit" title="}.encode_html($meta).qq{">}.encode_html($meta).qq{</a></span><span id="note-$basename-field-$id" style="display: none">} . $element . qq{<span class="buttons"><a href="#$id" title="$save_changes_label" class="button submit-note">$save_label</a><a href="#$id" type="button" class="button cancel-edit">$cancel_label</a></span></span>};
        }
    }
    return $out;
}

sub list_common {
    my ($cb, $app, $template) = @_;

    my $old = <<HTML;
\$checked.parents('tr').addClass('selected');
HTML
    $old = quotemeta($old);

    my $new = <<HTML;
//\$checked.parents('tr').addClass('selected');
HTML
    $$template =~ s/$old/$new/;

    $old = <<HTML;
jQuery('table.listing-table th :checkbox').prop('checked', true);
HTML
    $old = quotemeta($old);

    $new = <<HTML;
//jQuery('table.listing-table th :checkbox').prop('checked', true);
HTML
    $$template =~ s/$old/$new/;

}

sub add_css {
    my ($cb, $app, $template) = @_;

    my $q = $app->param;
    my $mode = $q->param('__mode');
    return if !($mode eq 'list' || $mode eq 'filtered_list' || $mode eq 'save_cf_from_list');

    my $type = $q->param('_type');
    return if !($type eq 'field');

    my $blog_id = $app->param('blog_id') || 0;

    my $plugin = $app->component('CustomFieldEditor');
    my $text_width = $plugin->get_config_value('text_width', 'system') || 100;
    my $textarea_width = $plugin->get_config_value('textarea_width', 'system') || 250;
    my $text = $text_width - 15;
    my $textarea = $textarea_width - 20;

    my @text_selectors;
    my @textarea_selectors;
    my @button_selectors;
    my @button_selectors2;
    foreach my $field (@fields) {
        if ( $field eq 'name' || $field eq 'tag' || $field eq 'basename' || $field eq 'default' || $field eq 'options') {
            push @text_selectors, ".col.head." . $field;
        } elsif ( $field eq 'description' ) {
            push @textarea_selectors, ".col.head." . $field;
        } elsif ( $field eq 'required' || $field eq 'type' || $field eq 'obj_type' ) {
            push @button_selectors, ".col.head." . $field;
            if ( $field eq 'radio' ) {
                push @button_selectors2, "td.col." . $field . " .buttons";
            }
        }
    }
    my $text_selector = join(",", @text_selectors) . "{width: ${text_width}px;}" if @text_selectors;
    my $textarea_selector = join(",", @textarea_selectors) . "{width: ${textarea_width}px;}" if @textarea_selectors;

    my $button_selector = join(",", @button_selectors) . "{width: 140px;}" if @button_selectors;
    my $button_selector2 = join(",", @button_selectors2) . "{display: block; margin-top: 3px;}" if @button_selectors2;

    my $old;
    my $new;

    $old = <<HTML;
<mt:var name="html_head">
HTML
    $old = quotemeta($old);
    $new = <<HTML;
<style type="text/css">
$text_selector
$textarea_selector
$button_selector
$button_selector2

.lfetext, .lfetextarea, .lfeurl {
    -moz-border-bottom-colors: none;
    -moz-border-image: none;
    -moz-border-left-colors: none;
    -moz-border-right-colors: none;
    -moz-border-top-colors: none;
    -moz-box-sizing: border-box;
    background-color: #FBFBFB;
    border-color: #ADADAD #DCDDDD #DCDDDD #ADADAD;
    border-radius: 3px 3px 3px 3px;
    border-right: 1px solid #DCDDDD;
    border-style: solid;
    border-width: 1px;
    box-shadow: 0 1px 0 #F3F3F3;
    font-size: 13px;
    outline: 0 none;
    padding: 0.2em 0.25em;
    vertical-align: middle;
    margin-bottom: 3px;
}
.lfetext, .lfeurl {
    width: ${text}px;
}
.lfetextarea {
    vertical-align: bottom;
    height: 100px;
    width: ${textarea}px;
}
.lfecheckbox {
    margin-bottom: 3px;
}
.lfeselect {
    margin-bottom: 3px;
}
.lferadio .submit-note, .lferadio .cancel-note {
    margin-bottom: 3px;
}
.image-link {
    margin: 3px 0;
}
</style>
  <mt:var name="html_head">
HTML

    $$template =~ s/$old/$new/;

    $old = <<HTML;
  <script type="text/javascript" src="<\$mt:var name="static_uri"\$>jqueryui/jquery-ui.js?v=<\$mt:var name="mt_version_id" escape="url"\$>"></script>
HTML
    $old = quotemeta($old);
    $new = <<HTML;
  <script type="text/javascript" src="<\$mt:var name="static_uri"\$>plugins/CustomFieldEditor/js/jquery-ui.js?v=<\$mt:var name="mt_version_id" escape="url"\$>"></script>
HTML
    $$template =~ s/$old/$new/;

    $old = <<HTML;
  <script type="text/javascript" src="<\$mt:var name="static_uri"\$>jqueryui/jquery-ui.min.js?v=<\$mt:var name="mt_version_id" escape="url"\$>"></script>
HTML
    $old = quotemeta($old);
    $new = <<HTML;
  <script type="text/javascript" src="<\$mt:var name="static_uri"\$>plugins/CustomFieldEditor/js/jquery-ui.js?v=<\$mt:var name="mt_version_id" escape="url"\$>"></script>
HTML
    $$template =~ s/$old/$new/;
}

sub is_limit {
    return (rest_day() > 0) ? 0 : 1;
}

sub rest_day {
    my $plugin = MT->component("CustomFieldEditor");
    my $field_column = $plugin->get_config_value('field_column', 'system') || '';
    my $duration = time() - $field_column;

    my $rest_day = 15 - int($duration / (24 * 3600));
    return ($rest_day > 0) ? $rest_day : 0;
}

sub save_cf_from_list {
    my $app = shift;

    return $app->forward('filtered_list') if is_limit();

    $app->validate_magic or return;
    my $author = $app->user;
    my $type = $app->param('_type') || 'field';

    my $class = $app->model($type)
        or return $app->errtrans("Invalid parameter");

    my $perms;
#    if ( $type eq 'field' ) {
#        $perms = $app->permissions
#            or return $app->permission_denied();
#    }

    my $id = $app->param('id');
    return if !$id;

    require MT::Blog;
    my $blog_id = $app->param('blog_id');
    if ( $blog_id ) {
        my $blog = MT::Blog->load($blog_id)
            or return $app->error(
            $app->translate( 'Can\'t load blog #[_1].', $blog_id ) );
    }
    my $obj = $class->load($id)
        || return $app->error(
        $app->translate( "No such [_1].", $class->class_label ) );

    my @param = $app->param;
    if (@param) {
        foreach my $key ( @param ) {
            $obj->$key( $app->param($key) ) if $obj->has_column( $key );
        }
    }

#    my $q = $app->param;
#    my $result = MT->model('field')->validates_uniqueness_of_tag(
#        $app,
#        {   tag      => $q->param('tag')      || undef,
#            blog_id  => $q->param('blog_id')  || undef,
#            id       => $q->param('id')       || undef,
#            basename => $q->param('basename') || undef,
#            obj_type => $q->param('obj_type') || undef,
#            type     => $q->param('type')     || undef,
#        }
#    );
#    return $result if $result != 1;

    $obj->save;

    return $app->forward('filtered_list');
}

sub edit_field {
    my ( $cb, $app, $param, $tmpl ) = @_;

    my $type = $app->param('_type');

    my $old_buf;
    my $link_buf;
    my $field_buf;
    my $note_buf;
    my $old_note_buf;
    my $empty_note_buf;
    my $empty_old_note_buf;
    my $recover_buf;
    my $clear_note_buf;
    my $new_note_buf;
    my @cancel_buf;
    my $submit_buf;
    my $render_buf;

    my @basenames;
    my $counter = 0;

    my $image_buf;
    my $old_image_buf;
    my $recover_image_buf;
    my $clear_image_buf;
    my $asset_ids;

    foreach my $field (@fields) {
        next if !( $field eq 'basename'    ||
                   $field eq 'default'     ||
                   $field eq 'description' ||
                   $field eq 'name'        ||
                   $field eq 'obj_type'    ||
                   $field eq 'options'     ||
                   $field eq 'required'    ||
                   $field eq 'options'     ||
                   $field eq 'tag'         ||
                   $field eq 'type' );
        $old_buf .=  "var old_note_" . $field . " = '';\nvar check_note_" . $field . " = '';\n";
        $link_buf .=  "note_links.push( jQuery('#note-" . $field . "-link-' + id) );\n";
        $field_buf .= "note_fields.push( jQuery('#note-" . $field . "-field-' + id) );\n";

        # note
        if ($field eq 'radio') {
            $note_buf .= "var note_" . $field . " = jQuery('[name=\"note-" . $field . "-' + id + '\"]:checked');\n";
        } else {
            $note_buf .= "var note_" . $field . " = jQuery('#note-" . $field . "-' + id);\n";
        }

        # old
        if ($field eq 'required') {
            $old_note_buf .= "old_note_" . $field . " =  jQuery('#note-" . $field . "-' + id).attr('checked');";
        } else {
            $old_note_buf .= "old_note_" . $field . " = note_" . $field . ".val();\ncheck_note_" . $field . " = note_" . $field . ".val();\n";
        }

        # empty
        if ($field ne 'required') {
            $empty_note_buf .= "if (note_" . $field . ".val() == '---'){\n  note_" . $field . ".val('');\n  check_note_" . $field . " = '';\n}\n";
        } else {
            $empty_note_buf .= "if (jQuery('#note-" . $field . "-' + id).attr('checked') == 'checked' || jQuery('#note-" . $field . "-' + id).attr('checked') == true) { note_" . $field . "= 1;\ncheck_note_" . $field . "= 1; } else { note_" . $field . "= 0;\ncheck_note_" . $field . "= 0; }";
        }

        # recover
        if ($field ne 'required') {
            $recover_buf .= "note_" . $field . ".val( old_note_" . $field . " );\n";
        } else {
            $recover_buf .= "note_" . $field . ".attr('checked', old_note_" . $field . ");\n";
        }
        $clear_note_buf .=  "old_note_" . $field . " = '';\n";

        # new
        if ($field eq 'required') {
            $new_note_buf .= "var new_note_" . $field . ";\nif (jQuery('#note-" . $field . "-' + id).attr('checked') == 'checked' || jQuery('#note-" . $field . "-' + id).attr('checked') == true) { new_note_" . $field . "= 1; } else { new_note_" . $field . "= 0; }";
        } else {
            $new_note_buf .= "var new_note_" . $field . " = note_" . $field . ".val();\n";
        }

        push @cancel_buf, "check_note_" . $field . " == new_note_" . $field;

        # submit
        if ($field eq 'radio') {
            $submit_buf .= "var note_" . $field . " = jQuery('[name=\"note-" . $field . "-' + id + '\"]:checked').val();\n";
        } elsif ($field eq 'required') {
            $submit_buf .= "var note_" . $field . ";\nif (jQuery('#note-" . $field . "-' + id)[0]) { if ( jQuery('#note-" . $field . "-' + id).attr('checked') == 'checked' || jQuery('#note-" . $field . "-' + id).attr('checked') == true) {note_" . $field . "= 1; } else { note_" . $field . "= 0; } } else { note_" . $field . "= undefined; }";
        } else {
            $submit_buf .= "var note_" . $field . " = jQuery('#note-" . $field . "-' + id).val();\n";
        }

        # render
        $render_buf .= $field . ": note_" . $field . ",\n";

        # asset_ids
#        $asset_ids = "var asset_ids = new Array();\nvar asset_ids_val = '';\njQuery('.include_asset_ids-' + id).each(function(){\nasset_ids[asset_ids.length] = jQuery(this).val();\n});\n
#if(asset_ids.length > 1){\nasset_ids_val = '[' + asset_ids.join(',') + ']';\n} else if(asset_ids.length == 1){\nasset_ids_val = asset_ids[0];\n} else {\nasset_ids_val = 0;\n}\n";
#
#        if ($field->type eq 'image') {
#            $old_image_buf .= "var old_image_" . $field . " = '';\n";
#            $image_buf .= "old_image_" . $field . " = jQuery('#image-" . $field . "-' + id).html();\n";
#            $recover_image_buf .= "if(old_note_" . $field . " == '---') {\nold_note_" . $field . " = '';\n}\njQuery('#image-" . $field . "-' + id).html( old_image_" . $field . " );\nif (old_image_" . $field . " == '---'){\n  jQuery('#delete-" . $field . "-' + id).css('display', 'none');\n} else {\n  jQuery('#delete-" . $field . "-' + id).css('display', 'inline');\n}\n";
#            $clear_image_buf .=  "old_image_" . $field . " = '';\n";
#        }

        $counter++;
    }
#    return 1 if !$counter;
    my $cancel = join(' && ', @cancel_buf);



    my $plugin = $app->component('CustomFieldEditor');
    my $field = $tmpl->getElementById('container');
    my $node = $tmpl->createElement('setvarblock', { name => 'jq_js_include', component => $plugin });

    my $html = <<HTML;
    var current_edit = 0;
    $old_buf
    $old_image_buf

    function clear_image(id, basename_id) {
        basename_id.match(/delete-(.*)/);
        var basename_id = RegExp.\$1;
        jQuery('#image-' + basename_id).html('---');
        jQuery('#note-' + basename_id).val('');
        jQuery('#include_asset_ids-' + basename_id).val('');
        jQuery('#delete-' + basename_id).css('display', 'none');
    }

    function edit_note(id) {
        if (current_edit) return;
        var note_links = new Array();
        $link_buf
        var note_fields = new Array();
        $field_buf
        if (!note_links.length || !note_fields.length) return;
        $note_buf
        current_edit = id;
        $old_note_buf
        $image_buf
        $empty_note_buf

        for ( i = 0; i < $counter; ++i ) {
            note_links[i].hide();
            note_fields[i].show();
        }
        return false;
    }

    function cancel_edit(id) {
        var note_links = new Array();
        $link_buf
        var note_fields = new Array();
        $field_buf
        $note_buf
        if (!note_links.length || !note_fields.length) return;
        $recover_image_buf
        $recover_buf
        $clear_image_buf
        $clear_note_buf
        current_edit = 0;

        for ( i = 0; i < $counter; ++i ) {
            note_links[i].show();
            note_fields[i].hide();
        }
        return false;
    }

    function submit_edit(id) {
        if ((0 != current_edit) && (id != current_edit))
            id = current_edit;
        $note_buf
        $new_note_buf

        if ($cancel) {
            cancel_edit(id);
            return false;
        }
        return submit_edit_form(id);
    }

    function submit_edit_form(id) {
        $submit_buf
        $asset_ids
        renderList({
HTML

    my $is_strict = 0;
    my $blog_id = $app->blog ? $app->blog->id : 0;
    $is_strict = $plugin->get_config_value('is_strict', 'blog:' . $app->blog->id ) if $blog_id;

    if ( $is_strict ) {
        $html .= "            __mode: 'save',\n";
    } else {
        $html .= "            __mode: 'save_cf_from_list',\n";
    }

    $html .= <<HTML;
            _type: 'field',
            id: id,
            $render_buf
            blog_id:jQuery('#input[name=blog_id]').val(),
            forward_list: 1,
            magic_token: 1
        });
        cancelCreateMode();
        return false;
    }

    jQuery(window).bind('listReady', function() {
        jQuery('a.mt-open-dialog').mtDialog();
        jQuery('.clear-image').each(function () {
            jQuery(this).attr('href').match(/#(\\d+)/);
            var id = RegExp.\$1;
            jQuery(this).attr('id').match(/delete-(.*)/);
            var basename_id = RegExp.\$1;
            if (jQuery('#image-' + basename_id).html() == '---'){
                jQuery(this).css('display', 'none');
            } else {
                jQuery(this).css('display', 'inline');
            }
        });


        jQuery('.start-edit').on('click', function () {
            jQuery(this).attr('href').match(/#(\\d+)/);
            var id = RegExp.\$1;
            cancel_edit(current_edit);
            edit_note(id);
            return false;
        });

        jQuery('.cancel-edit').on('click', function () {
            jQuery(this).attr('href').match(/#(\\d+)/);
            var id = RegExp.\$1;
            cancel_edit(id);
            return false;
        });

        jQuery('.submit-note').on('click', function () {
            jQuery(this).attr('href').match(/#(\\d+)/);
            var id = RegExp.\$1;
            submit_edit(id);
            return false;
        });

        jQuery('.clear-image').on('click', function () {
            jQuery(this).attr('href').match(/#(\\d+)/);
            var id = RegExp.\$1;
            var basename = jQuery(this).attr('id');
            clear_image(id, basename);
            return false;
        });
    });

HTML

    $node->innerHTML($html);
    $tmpl->insertBefore($node, $field);

#    my $node2 = $tmpl->createElement('setvarblock', { name => 'listing_footer_content', append => 1 });
#    my $html2 = 'Can use ' . rest_day() . 'days';
#    $node2->innerHTML($html2);
#    $tmpl->insertBefore($node2, $field);

    my $q      = $app->param;
    my $plugin = $app->component("Commercial");

    my $blog = $app->blog;
    my $blog_id = $blog ? $blog->id : 0;
    return $app->permission_denied()
        if !$app->user->is_superuser && !$blog_id;
    return $app->permission_denied() if !$app->can_do('administer_blog');
    my $id = $app->param('id');

    my ( @obj_types, @types_loop );

    require CustomFields::Field;
    for my $key ( @{ CustomFields::Field->column_names() } ) {
        if ( my $val = $q->param($key) ) {
            $param->{$key} = $val;
        }
    }
    $param->{object_label} = CustomFields::Field->class_label;

    my $obj_type = $q->param('obj_type');

    if ($id) {
        my $field
            = CustomFields::Field->load( { blog_id => $blog_id, id => $id } );
        return $app->return_to_dashboard( redirect => 1 ) if !$field;
        $obj_type ||= $field->obj_type;
        while ( my ( $key, $val ) = each %{ $field->column_values() } ) {
            $param->{$key} ||= $val;
        }
        $param->{basename_old} = $field->basename;
    }

    $param->{basename_limit} = ( $blog ? $blog->basename_limit : 0 ) || 30;

    my $customfield_objs = $app->registry('customfield_objects');
    my @cf_obj_keys      = sort {
        ( $customfield_objs->{$a}{order} || 0 )
            <=> ( $customfield_objs->{$b}{order} || 0 )
        }
        keys %$customfield_objs;

    foreach my $key (@cf_obj_keys) {
        my $context = $customfield_objs->{$key}->{context};
        if ( ref $context ) {
            if ($blog) {
                next
                    if $blog->is_blog && !( grep { $_ eq 'blog' } @$context );
                next
                    if !$blog->is_blog
                        && !( grep { $_ eq 'website' } @$context );
            }
            else {
                next unless grep { $_ eq 'system' } @$context;
            }
        }
        elsif ( $context eq 'system' && $blog_id ) {
            next;
        }
        elsif ( $context eq 'blog' && !$blog_id ) {
            next;
        }

        my $class = $app->model($key);
        push @obj_types,
            {
            obj_type       => $key,
            obj_type_label => ucfirst( $class->class_label ),
            ( $obj_type && $key eq $obj_type ) ? ( selected => 1 ) : (),
            };
    }

    my $customfield_types = $app->registry('customfield_types');
    my @customfield_types_loop;

    # Resort it by the order key
    my @cf_types = sort {
        ( $customfield_types->{$a}{order} || 0 )
            <=> ( $customfield_types->{$b}{order} || 0 )
        }
        keys %$customfield_types;

    foreach my $key (@cf_types) {
        next if ref $key eq 'HASH';
        my $type = $customfield_types->{$key};

        my $context = $customfield_types->{$key}->{context} || '';
        if ( ref $context ) {
            if ($blog) {
                next
                    if $blog->is_blog && !( grep { $_ eq 'blog' } @$context );
                next
                    if !$blog->is_blog
                        && !( grep { $_ eq 'website' } @$context );
            }
            else {
                next unless grep { $_ eq 'system' } @$context;
            }
        }
        elsif ( $context eq 'system' && $blog_id ) {
            next;
        }
        elsif ( $context eq 'blog' && !$blog_id ) {
            next;
        }

       # This $tmpl_param is used to build the default field and options field
        my $tmpl_param = $param;
        $tmpl_param->{key}        = $key;
        $tmpl_param->{field_name} = 'default';
        $tmpl_param->{field_id}   = 'default';
        $tmpl_param->{field_value}
            = $id
            ? $param->{default}
            : ( $param->{default} || $type->{default_value} );
        $tmpl_param->{options} = $param->{options};

      # If an options_delimiter is present, we need to populate an option_loop
        if ( $type->{options_delimiter} && $param->{options} ) {
            my @option_loop;
            my $expr = '\s*' . $type->{options_delimiter} . '\s?';
            my @options = split /$expr/, $param->{options};
            foreach my $option (@options) {
                my $label = $option;
                if ( $option =~ m/=/ ) {
                    ( $option, $label ) = split /\s*=\s*/, $option, 2;
                }
                my $option_row = { option => $option, label => $label };
                $option_row->{is_selected}
                    = defined $tmpl_param->{field_value}
                    ? ( $tmpl_param->{field_value} eq $option )
                    : 0;
                push @option_loop, $option_row;
            }
            $tmpl_param->{option_loop} = \@option_loop;
        }

        my $row = {
            key   => $key,
            label => $type->{label},
            $type->{no_default} ? ()
            : ( default_field => _get_html( $key, 'field_html', $tmpl_param )
            ),
            $type->{options_field}
            ? ( options_field =>
                    _get_html( $key, 'options_field', $tmpl_param ) )
            : (),
            options_delimiter => $type->{options_delimiter},
            show_column =>
                ( defined $type->{show_column} ? $type->{show_column} : 1 ),
            (   defined $type->{asset_type}
                ? ( asset_type => $type->{asset_type} )
                : ()
            ),
        };

        foreach my $k ( keys %$row ) {
            my $value = $row->{$k};
            if ( ref($value) eq 'CODE' ) {    # handle coderefs
                $row->{$k} = $value->(@_);
            }
        }

        push @customfield_types_loop, $row;
    }

    $param->{customfield_types_loop} = \@customfield_types_loop;
    $param->{obj_type_loop}          = \@obj_types;

    eval { require MT::Image; MT::Image->new or die; };
    $param->{do_thumb} = !$@ ? 1 : 0;
    $param->{saved} = $app->param('saved') || 0;
    $param->{cfg_customfield} = 1;
    $param->{level_label}
        = $blog
        ? (
          $blog->is_blog
        ? $app->translate('blog')
        : $app->translate('website')
        )
        : $app->translate('Movable Type');

    my $cat_type;
    if (   $blog_id
        && $id
        && ( ( $obj_type eq 'entry' ) || ( $obj_type eq 'page' ) ) )
    {
        my $entry_class = $app->model($obj_type);
        my $cat_class   = $app->model( $entry_class->container_type );
        my $cats        = $app->_build_category_list(
            blog_id => $blog_id,
            type    => $entry_class->container_type
        );
        foreach (@$cats) {
            if ( grep { $_ == $id } @{ $_->{category_fields} } ) {
                $_->{selected} = 1;
            }
        }

        $param->{categories}             = $cats;
        $param->{container_type}         = $entry_class->container_type;
        $param->{container_label_plural} = $cat_class->class_label_plural;
    }

    $app->add_breadcrumb(
        $app->translate("Custom Fields"),
        $app->uri(
            'mode' => 'list_field',
            args   => { $blog_id ? ( blog_id => $blog_id ) : () }
        )
    );

    $app->add_breadcrumb( $app->translate('Edit Field') );
    $app->build_page( $plugin->load_tmpl('edit_field.tmpl'), $param );
}

sub field_list_header {
    my ($cb, $app, $template) = @_;

    my $old;
    my $new;

    $old = <<HTML;
</mt:setvarblock>
HTML
    $old = quotemeta($old);

    $new = <<HTML;
</mt:setvarblock>
<mt:setvarblock name="system_msg" append="1">
<mt:if name="request.error">
  <mtapp:statusmsg
     id="generic-error"
     class="error"
     can_close="0">
    <mt:var name="error">
  </mtapp:statusmsg>
</mt:if>
<mt:if name="request.saved">
  <mtapp:statusmsg
     id="saved"
     class="success">
    <__trans phrase="You have added [_1] to your address book." params="<strong><\$mt:var name="saved" escape="html"\$></strong>">
  </mtapp:statusmsg>
</mt:if>
<mt:if name="request.saved_deleted">
  <mtapp:statusmsg
     id="saved-deleted"
     class="success">
    <__trans phrase="You have successfully deleted the selected contacts from your address book.">
  </mtapp:statusmsg>
</mt:if>
</mt:setvarblock>

<mt:setvarblock name="content_header" append="1">

<form class="create-inline" id="create-inline-field" style="display: none;">

<mt:setvarblock name="system_msg">
<mt:if name="saved">
  <mtapp:statusmsg
     id="saved"
     class="success"
     can_close="1">
    <__trans phrase="Your changes have been saved.">
  </mtapp:statusmsg>
</mt:if>
<mt:if name="saved_deleted">
  <mtapp:statusmsg
     id="saved-deleted"
     class="success"
     can_close="1">
    <__trans phrase="The selected field(s) has been deleted from the database.">
  </mtapp:statusmsg>
</mt:if>
<mt:if name="error">
  <mtapp:statusmsg
     id="generic-error"
     class="error"
     can_close="0">
    <mt:var name="error" escape="html">
  </mtapp:statusmsg>
</mt:if>
<mtapp:statusmsg
   id="blank-fields"
   class="error hidden"
   can_close="0">
  <__trans phrase="You must enter information into the required fields highlighted below before the custom field can be created.">
</mtapp:statusmsg>
</mt:setvarblock>

<mt:setvarblock name="html_head">
<link rel="stylesheet" href="<mt:var name="static_uri">addons/Commercial.pack/styles-customfields.css" type="text/css" media="screen" title="CustomFields Stylesheet" charset="utf-8" />
<script type="text/javascript">
<!--
    var customfield_types_loop = <mt:var name="customfield_types_loop" to_json="1">

    function htmlEscape( val ) {
      return val.replace(/&/g, "&amp;").replace(/"/g, "&quot;").replace(/</g, "&lt;").replace(/>/g, "&gt;");
    }

    function toggleClassName(el, cn) {
        if (DOM.hasClassName(el, cn))
            DOM.removeClassName(el, cn);
        else
            DOM.addClassName(el, cn);
    }

    function toggleCode(el) {
        var name = htmlEscape( getByID('name').value );
        var code = getByID('code');
        var tag = htmlEscape( getByID('tag').value );
        var text = '&lt;mt:If tag="' + tag + '"&gt;<br />' + name + ': &lt;mt:' + tag + '&gt;<br />&lt;/mt:If&gt;';
        code.innerHTML = text;

        var opt = getByID(el);
        if (opt) {
            toggleClassName(opt, 'active');
        }
    }

    function toggleFile() {
        var fld = getByID("basename");
        if (fld) {
            fld.disabled = false;
            fld.focus();
            var baseman = getByID("basename_manual");
            if (baseman) baseman.value = "1";
            var basewarn = getByID("basename-warning");
            if (basewarn) basewarn.style.display = "block";
        }
        var img = getByID("basename-lock");
        if (img)
            img.style.display = 'none';
        return false;
    }

    function rebasename(title) {
        if (!orig_basename) {
            dir_title = dirify(title.value);
//            dir_title = dir_title.substring(0, <\$mt:var name="basename_limit"\$>);
            trimmed = dir_title.match(/^(.*[^_])/);
            if (trimmed && trimmed.length) {
                setElementValue('basename', trimmed[0]);
            } else {
                setElementValue('basename', '');
            }
        }
    }

    function capitalizeMe(val) {
            newVal = '';
            val = val.split(' ');
            for(var c=0; c < val.length; c++) {
                    newVal += val[c].substring(0,1).toUpperCase() +
                    val[c].substring(1,val[c].length);
            }
            return newVal.replace(/^\s+/g, '').replace(/\s+\$/g, '');
    }

    var cloneTypes;
    function changeTypeByObject() {
<mt:unless name="id">
       var obj = jQuery('#obj_type');
       if ( obj[0].value == 'comment' ) {
           if (!cloneTypes) {
               cloneTypes = jQuery('.cf-type-asset').clone();
               jQuery('.cf-type-asset').remove();
           }
       } else {
           if (cloneTypes) {
               jQuery('#type').append(cloneTypes);
               cloneTypes = null;
           }
       }
</mt:unless>
    }

    function changeName(t) {
        <mt:unless name="id">
        var opt = getByID('tag');
        var name = dirify(getByID('name').value);
        name = name.replace(/-/g, '');
        var tag = getByID('tag').value;
        var obj_type = getByID('obj_type').value;

        if(!t)
            tag = getByID('tag').value = capitalizeMe(obj_type) + 'Data' + capitalizeMe(name);
        </mt:unless>
    }

    function changeType(s) {
        if(!s)
            s = document.getElementById('type');

        <mt:if name="new_object">
        var type_key = s.options[s.selectedIndex].value;
        <mt:else>
        var type_key = s.value;
        </mt:if>

        var type_obj;
        for (var i = 0; i < customfield_types_loop.length; i++) {
            if(customfield_types_loop[i].key == type_key) {
                type_obj = customfield_types_loop[i];
                break;
            }
        }

        if(!type_obj)
            return;

        // Set options_field
        if(type_obj.options_field) {
            show('options-field');
            var optField = DOM.getElementsByClassName(getByID('options-field'), 'field-content')[0];
            optField.innerHTML = type_obj.options_field;
        } else {
            hide('options-field');
        }

        // Set default_field
        if(type_obj.default_field) {
            show('default-field');
            var optField = DOM.getElementsByClassName(getByID('default-field'), 'field-content')[0];

            // If it's got some options that can be set, the default will need to be set after a save
            if(!type_obj.options_field || "<mt:var name="type" escape="js">" == type_key) {
                hide('default-save');
                optField.innerHTML = type_obj.default_field;
            } else {
                hide('default-field');
                show('default-save');
            }
        } else {
            hide('default-field');
            hide('default-save');
        }
    }

    function checkRequired(f) {
        var fields = jQuery('#main-content div.required :input:not(:button):visible')
                         .map(function() {
                             return this.id;
                         });
        var submit = true;
        for (var i = 0; i < fields.length; i++) {
            var field = document.getElementById(fields[i]);
            if(!field.value || field.value == '') {
                if(DOM.hasClassName(field.parentNode, 'textarea-wrapper'))
                    field.parentNode.style.borderColor = '#FF0000';
                else
                    field.style.borderColor = '#FF0000';
                field.focus();
                submit = false;
                DOM.removeClassName(getByID('blank-fields'), 'hidden');
            } else {
                if(DOM.hasClassName(field.parentNode, 'textarea-wrapper'))
                    field.parentNode.style.borderColor = '#DADADA';
                else
                    field.style.borderColor = '#DADADA';
            }
        }

        return submit;
    }

    var orig_basename;

    function init() {
        changeType();
        changeName();
        var basename = getByID("basename");
        var basename_old = getByID("basename_old");
        var title = getByID("title");
        if (basename) {
            if (basename.value != '') {
                <mt:if name="reedit">
                toggleFile();
                </mt:if>
                orig_basename = basename.value;
            }<mt:if name="reedit"> else {
                <mt:if name="id">
                basename.value = basename_old.value;
                <mt:else>
                basename.value = dirify(title.value);
                </mt:if>
            }</mt:if>
            <mt:unless name="id">
            basename.disabled = false;
            </mt:unless>
        }
    }

    TC.attachLoadEvent(init);
//-->
</script>
</mt:setvarblock>

  <mtapp:statusmsg
     id="default-save"
     class="info">
    <__trans phrase="You must save this custom field before setting a default value.">
  </mtapp:statusmsg>

<!--<form method="post" action="<mt:var name="script_url">" onsubmit="return checkRequired(this);">-->
  <input type="hidden" name="__mode" value="save" id="__mode" />
  <input type="hidden" name="_type" value="field" />
  <input type="hidden" name="blog_id" value="<mt:var name="blog_id" default="0">" />
<mt:if name="id">
  <input type="hidden" name="id" value="<mt:var name="id" escape="html">" />
</mt:if>
  <input type="hidden" name="magic_token" value="<mt:var name="magic_token">" id="magic_token" />
  <input type="hidden" name="return_args" value="<mt:var name="return_args">" id="return_args" />

  <mtapp:setting
     id="obj_type"
     label="<__trans phrase="System Object">"
     required="1"
     hint="<__trans phrase="Choose the system object where this custom field should appear.">"
     show_hint="\$new_object">
  <mt:if name="id">
    <mt:loop name="obj_type_loop">
      <mt:if name="selected">
    <input type="hidden" name="obj_type" id="obj_type" value="<mt:var name="obj_type">" />
    <mt:var name="obj_type_label" escape="html">
      </mt:if>
    </mt:loop>
  <mt:else>
    <select name="obj_type" id="obj_type" onchange="changeName(); changeTypeByObject()">
      <option value=""><__trans phrase="Select..."></option>
    <mt:loop name="obj_type_loop">
      <option value="<mt:var name="obj_type">"<mt:if name="selected"> selected="selected"</mt:if>><mt:var name="obj_type_label" escape="html"></option>
    </mt:loop>
    </select>
  </mt:if>
  </mtapp:setting>

  <mtapp:setting
     id="name"
     label="<__trans phrase="Name">"
     required="1">
<!--    <input type="text" name="name" value="<mt:var name="name" escape="html">" id="name" class="text" onchange="rebasename(this); changeName();" />-->
    <input type="text" name="name" value="" id="name" class="text" />
  </mtapp:setting>

  <mtapp:setting
     id="description"
     label="<__trans phrase="Description">">
    <textarea id="description" name="description" class="text low"><mt:var name="description" escape="html"></textarea>
  </mtapp:setting>

  <mtapp:setting
     id="type"
     label="<__trans phrase="Type">"
     required="1">
  <mt:if name="id">
    <mt:loop name="customfield_types_loop">
      <mt:if name="type" eq="\$key">
    <input type="hidden" name="type" id="type" value="<mt:var name="key">" />
    <mt:var name="label" escape="html">
      </mt:if>
    </mt:loop>
  <mt:else>
    <select name="type" onchange="changeType(this);" id="type">
    <mt:loop name="customfield_types_loop">
      <mt:if name="show_column">
      <option<mt:if name="asset_type"> class="cf-type-asset"</mt:if> value="<mt:var name="key">"<mt:if name="type" eq="\$key"> selected="selected"</mt:if>><mt:var name="label" escape="html"></option>
      </mt:if>
    </mt:loop>
    </select>
  </mt:if>
  </mtapp:setting>

  <mtapp:setting
     id="options"
     label="<__trans phrase="Options">"
     required="1">
  </mtapp:setting>

  <mtapp:setting
     id="required"
     label="<__trans phrase="Required?">"
     hint="<__trans phrase="Is data entry required in this custom field?">"
     show_hint="0">
    <input type="checkbox" name="required" value="1" id="required"<mt:if name="required"> checked="checked"</mt:if> /> <__trans phrase="Must the user enter data into this custom field before the object may be saved?">
  </mtapp:setting>

  <mtapp:setting
     id="default"
     label="<__trans phrase="Default">">
  </mtapp:setting>

  <mtapp:setting
     id="basename"
     label="<__trans phrase="_CF_BASENAME">"
     required="1"
     hint='<__trans phrase="The basename must be unique within this [_1]." params="<mt:var name="level_label">">'
     show_hint="1">
    <input type="hidden" name="basename_manual" id="basename_manual" value="0" />
    <input type="hidden" name="basename_old" id="basename_old" value="<mt:if name="id"><mt:var name="basename_old" escape="html"></mt:if>" />
<!--    <input type="text" name="basename" id="basename" class="text mt-edit-field<mt:if name="error"> show-input</mt:if>" maxlength="65" value="<mt:var name="basename" escape="html">" onchange="setElementValue('basename_manual', '1');" />-->
    <input type="text" name="basename" id="basename" class="text mt-edit-field<mt:if name="error"> show-input</mt:if>" maxlength="65" value="" onchange="setElementValue('basename_manual', '1');" />
    <p class="alert-warning-inline icon-left icon-warning" id="basename-warning" style="display: none;"><__trans phrase="Warning: Changing this field's basename may require changes to existing templates."></p>
  </mtapp:setting>

  <mtapp:setting
     id="tag"
     label="<__trans phrase="Template Tag">"
     required="1">
    <input type="text" name="tag" value="<mt:var name="tag" escape="html">" id="tag" class="text" onkeyup="changeName(1);" />
    <div id="example-tmplate-code">
      <a href="#bottom" onclick="toggleCode('tag-field');" class="toggle-link icon-left icon-spinner"><__trans phrase="Example Template Code"></a>
      <pre class="code" id="code"></pre>
    </div>
  </mtapp:setting>

  <mt:if name="blog_id">
    <mt:if name="categories">
  <h3><mt:var name="container_label_plural"></h3>
  <mtapp:setting
     id="show_fields"
     label='<__trans phrase="Show In These [_1]" params="<mt:var name="container_label_plural">">'>
    <input type="hidden" name="container_type" value="<mt:var name="container_type">" />
    <ul>
    <mt:loop name="categories">
      <li style="margin-left: <mt:var name="category_pixel_depth">px;"><label for="category_<mt:var name="category_id">"><input type="checkbox" name="show_field_<mt:var name="category_id">" id="category_<mt:var name="category_id">" value="1"<mt:if name="selected"> checked="checked"</mt:if> /> <mt:var name="category_label" escape="html"></label><input type="hidden" name="selected_fields_<mt:var name="category_id">" value="<mt:var name="category_selected_fields" escape="html">" /></li>
    </mt:loop>
    </ul>
  </mtapp:setting>
    </mt:if>
  </mt:if>

    <div class="actions-bar settings-actions-bar">
        <button
           type="submit"
           id="submit-new-contact"
           class="button action primary">
          <__trans phrase="Add">
        </button>
        <button
           type="button"
           class="button action mt-cancel-action">
          <__trans phrase="Cancel">
        </button>
    </div>
</form>
</mt:setvarblock>

<mt:setvarblock name="jq_js_include" append="1">

    var current_edit = 0;
    var old_note_email = '';
    var old_note_url = '';

    function edit_note_new(id) {
        if (current_edit) return;
        var note_links = new Array();
        note_links.push( jQuery('#note-email-link-' + id) );
        note_links.push( jQuery('#note-url-link-'   + id) );
        var note_fields = new Array();
        note_fields.push( jQuery('#note-email-field-' + id) );
        note_fields.push( jQuery('#note-url-field-'   + id) );


        note_fields.push( jQuery('#name') );
        note_fields.push( jQuery('#type') );
        note_fields.push( jQuery('#description') );

        var \$note_email = jQuery('#note-email-' + id);
        var \$note_url   = jQuery('#note-url-'   + id);
        if (!note_links.length || !note_fields.length || !\$note_email) return;
        current_edit = id;

        old_note_email = \$note_email.val();
        old_note_url   = \$note_url.val();
        for (i=0;i<2;++i) {
            note_links[i].hide();
            note_fields[i].show();
        }
        \$note_email.focus();
        \$note_email.select();
        return false;
    }

    function cancel_edit_new(id) {
        var note_links = new Array();
        note_links.push( jQuery('#note-email-link-' + id) );
        note_links.push( jQuery('#note-url-link-'   + id) );
        var note_fields = new Array();
        note_fields.push( jQuery('#note-email-field-' + id) );
        note_fields.push( jQuery('#note-url-field-'   + id) );

        var \$note_email = jQuery('#note-email-' + id);
        var \$note_url   = jQuery('#note-url-'   + id);
        if (!note_links.length || !note_fields.length || !\$note_email) return;

        \$note_email.val( old_note_email );
        old_note_email = '';
        \$note_url.val( old_note_url );
        old_note_url = '';
        current_edit = 0;
        for (i=0;i<2;++i) {
            note_links[i].show();
            note_fields[i].hide();
        }
        \$note_email.mtUnvalidate();
        \$note_url.mtUnvalidate();
        return false;
    }

    function setCreateMode () {
        jQuery('#create-inline-field').show();
        jQuery('#url').val('');
        jQuery('#email').val('').focus();
        jQuery('#content-actions').hide();
    }

    function cancelCreateMode () {
        jQuery('#create-inline-field').hide();
        jQuery('#create-inline-field input').mtUnvalidate();
        jQuery('#content-actions').show();

        current_edit = 0;
    }

    jQuery('#submit-new-contact').click(function() {
        if ( !jQuery('#create-inline-field input').mtValidate('simple') )
            return false;

        if ( jQuery('#required').prop('checked') == true ) {
            required = 1;
        } else {
            required = 0;
        }
        renderList({
            __mode: 'save',
            _type: 'field',
            blog_id: jQuery('#input[name=blog_id]').val(),
            obj_type: jQuery('#obj_type option:selected').val(),
            name: jQuery('#name').val(),
            description: jQuery('#description').val(),
            type: jQuery('#type option:selected').val(),
            options: jQuery('#options').val(),
            required: required,
            default:  jQuery('#default').val(),
            basename: jQuery('#basename').val(),
            tag: jQuery('#tag').val(),
            forward_list: 1,
            magic_token: 1
        });
        cancelCreateMode();
        return false;
    });

    jQuery('#create-inline-field button.mt-cancel-action').click(function() {
        cancelCreateMode();
        return false;
    });

    jQuery('#action-create-field').click( function() {
        setCreateMode();
        return false;
    });
    
//    jQuery('#listing-table')
//      .on('click','.start-edit',function(){
//        jQuery(this).attr('href').match(/#(\d+)/);
//        var id = RegExp.\$1;
//        edit_note_new(id);
//        return false;
//      })
//      .on('click','.cancel-edit',function(){
//        jQuery(this).attr('href').match(/#(\d+)/);
//        var id = RegExp.\$1;
//        cancel_edit_new(id);
//        return false;
//      })
//      .on('click','.submit-note',function(){
//        jQuery(this).attr('href').match(/#(\d+)/);
//        var id = RegExp.\$1;
//        submit_edit_new(id);
//        return false;
//      });

</mt:setvarblock>
HTML

    $$template =~ s/$old/$new/;

}

sub get_obj_types {
    my $app = shift;
    my $blog = $app->blog;
    my $blog_id = $blog ? $blog->id : 0;

    my $customfield_objs = $app->registry('customfield_objects');
    my @cf_obj_keys      = sort {
        ( $customfield_objs->{$a}{order} || 0 )
            <=> ( $customfield_objs->{$b}{order} || 0 )
        }
        keys %$customfield_objs;

    my @obj_types;
    foreach my $key (@cf_obj_keys) {
        my $context = $customfield_objs->{$key}->{context};
        if ( ref $context ) {
            if ($blog) {
                next
                    if $blog->is_blog && !( grep { $_ eq 'blog' } @$context );
                next
                    if !$blog->is_blog
                        && !( grep { $_ eq 'website' } @$context );
            }
            else {
                next unless grep { $_ eq 'system' } @$context;
            }
        }
        elsif ( $context eq 'system' && $blog_id ) {
            next;
        }
        elsif ( $context eq 'blog' && !$blog_id ) {
            next;
        }

        my $class = $app->model($key);
        push @obj_types,
            {
            obj_type       => $key,
            obj_type_label => ucfirst( $class->class_label ),
#            ( $obj_type && $key eq $obj_type ) ? ( selected => 1 ) : (),
            };
    }
    \@obj_types;
}

sub get_customfield_types {
    my $app = shift;
    my $blog = $app->blog;
    my $blog_id = $blog ? $blog->id : 0;

    my $customfield_types = $app->registry('customfield_types');
    my @customfield_types_loop;

    # Resort it by the order key
    my @cf_types = sort {
        ( $customfield_types->{$a}{order} || 0 )
            <=> ( $customfield_types->{$b}{order} || 0 )
        }
        keys %$customfield_types;

    foreach my $key (@cf_types) {
        next if ref $key eq 'HASH';
        my $type = $customfield_types->{$key};

        my $context = $customfield_types->{$key}->{context} || '';
        if ( ref $context ) {
            if ($blog) {
                next
                    if $blog->is_blog && !( grep { $_ eq 'blog' } @$context );
                next
                    if !$blog->is_blog
                        && !( grep { $_ eq 'website' } @$context );
            }
            else {
                next unless grep { $_ eq 'system' } @$context;
            }
        }
        elsif ( $context eq 'system' && $blog_id ) {
            next;
        }
        elsif ( $context eq 'blog' && !$blog_id ) {
            next;
        }

       # This $tmpl_param is used to build the default field and options field
#        my $tmpl_param = $param;
#        $tmpl_param->{key}        = $key;
#        $tmpl_param->{field_name} = 'default';
#        $tmpl_param->{field_id}   = 'default';
#        $tmpl_param->{field_value}
#            = $id
#            ? $param->{default}
#            : ( $param->{default} || $type->{default_value} );
#        $tmpl_param->{options} = $param->{options};
#
#      # If an options_delimiter is present, we need to populate an option_loop
#        if ( $type->{options_delimiter} && $param->{options} ) {
#            my @option_loop;
#            my $expr = '\s*' . $type->{options_delimiter} . '\s?';
#            my @options = split /$expr/, $param->{options};
#            foreach my $option (@options) {
#                my $label = $option;
#                if ( $option =~ m/=/ ) {
#                    ( $option, $label ) = split /\s*=\s*/, $option, 2;
#                }
#                my $option_row = { option => $option, label => $label };
#                $option_row->{is_selected}
#                    = defined $tmpl_param->{field_value}
#                    ? ( $tmpl_param->{field_value} eq $option )
#                    : 0;
#                push @option_loop, $option_row;
#            }
#            $tmpl_param->{option_loop} = \@option_loop;
#        }

        my $row = {
            key   => $key,
            label => $type->{label},
#            $type->{no_default} ? ()
#            : ( default_field => _get_html( $key, 'field_html', $tmpl_param )
#            ),
#            $type->{options_field}
#            ? ( options_field =>
#                    _get_html( $key, 'options_field', $tmpl_param ) )
#            : (),
#            options_delimiter => $type->{options_delimiter},
#            show_column =>
#                ( defined $type->{show_column} ? $type->{show_column} : 1 ),
#            (   defined $type->{asset_type}
#                ? ( asset_type => $type->{asset_type} )
#                : ()
#            ),
        };

        foreach my $k ( keys %$row ) {
            my $value = $row->{$k};
            if ( ref($value) eq 'CODE' ) {    # handle coderefs
                $row->{$k} = $value->(@_);
            }
        }

        push @customfield_types_loop, $row;
    }
    \@customfield_types_loop;
}
1;
